/*
 @C:\Web_545\Sql\End_Conv\Oracle\1065_FINSTPO_GERA_AJUSTE_NFDEVOLUCAO.SQL

 Descrio:
 Procedure para que ao aprovar uma nota de devoluo seja gerado desconto no boleto

 Parmetros

 V_CDEMPRESA    - Cdigo da empresa
 V_IDENTRSAIDOP - Verifica se a nota fiscal  de entrada ou sada
 V_RETORNO      - Retorno da procedure CANCELA_DADOS_FINANCEIRO
 V_VRDISPONIVEL - Valor disponvel da nota fiscal de devoluo
 V_VRAJUSTE     - Valor do ajuste
 V_CDAJUCTPAG   - Cdigo do tipo de ajuste de conta a Pagar
 V_CDAJUCTREC   - Cdigo do tipo de ajuste de conta a Receber
 V_VRDEVOLUTI   - Valor devolvido
 V_EXISTEAJUS   - Verifica se existe ajuste no boleto
 V_DEVOLUCAOREC - Devoluo de uma conta j paga em conta corrente
 V_DEVOLUCAOREC - Gera um cdigo sequncial na devoluo
 V_DEVOLUCAOPAG - Gera um cdigo sequncial na devoluo

 raise_application_error(-20501, '' , True);
 */

CREATE OR REPLACE PROCEDURE GERA_AJUSTE_NFDEVOLUCAO(P_CDFILIAL   IN VARCHAR2,
                                                    P_NRLANCTONF IN VARCHAR2)
AS
   V_CDEMPRESA     VARCHAR2(02);
   V_IDENTRSAIDOP  VARCHAR2(01);
   V_CDAJUCTPAG    VARCHAR2(04);
   V_CDAJUCTREC    VARCHAR2(04);
   V_DEVOLUCAOPAG  VARCHAR2(20);
   V_DEVOLUCAOREC  VARCHAR2(20);
   V_VRDISPONIVEL  FLOAT;
   V_VRAJUSTE      FLOAT;
   V_VRDEVOLUTI    FLOAT;
   V_EXISTEAJUS    NUMBER;
   V_RETORNO       NUMBER;
BEGIN

   V_VRDEVOLUTI := 0;

   SELECT NF.IDENTRSAIDOP, NF.CDEMPRESA, ROUND((NF.VRNOTAFISC - NVL(NF.VRDEVOLNFUTL, 0)), 2) AS VRDISPONIVEL
     INTO V_IDENTRSAIDOP, V_CDEMPRESA, V_VRDISPONIVEL
     FROM NOTAFISCAL NF
    WHERE NF.CDFILIAL   = P_CDFILIAL
      AND NF.NRLANCTONF = P_NRLANCTONF;

   BEGIN
      /* Verifica se est parametrizado o Tipo de Ajuste */
      SELECT CDAJUCTPAG, CDAJUCTREC
        INTO V_CDAJUCTPAG, V_CDAJUCTREC
        FROM PARAMEMPRESA
       WHERE CDEMPRESA = V_CDEMPRESA;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CDAJUCTPAG := NULL;
   END;

   IF V_IDENTRSAIDOP = 'S' THEN
      IF NOT V_CDAJUCTPAG IS NULL THEN
         /* Verifca a nota fiscal de origem */
         FOR C_NOTAS IN (SELECT CDFILIALIT, NRLANCTONFIT
                           FROM ITEMNOTAFISC
                          WHERE CDFILIAL   = P_CDFILIAL
                            AND NRLANCTONF = P_NRLANCTONF) LOOP

            CANCELA_DADOS_FINANCEIRO(C_NOTAS.CDFILIALIT,
                                     C_NOTAS.NRLANCTONFIT,
                                     'N',
                                     V_RETORNO);

            /* RETORNO = 0 - CANCELAMENTO FINANCEIRO CONCLUDO
               RETORNO = 4 - BOLETO COM AJUSTE */
            IF (V_RETORNO = 0 OR V_RETORNO = 4) THEN
               FOR C_CONTAS IN (SELECT BO.CDFILIAL, BO.NRLANBOLPAG, ROUND(BO.VRFINALBPAG, 2) AS VRFINALBPAG
                                  FROM CONTAPAGAR CP, BOLETPAGAR BO
                                 WHERE CP.CDFILIAL    = C_NOTAS.CDFILIALIT
                                   AND CP.NRLANCTONF  = C_NOTAS.NRLANCTONFIT
                                   AND CP.CDFILIBPAG  = BO.CDFILIAL
                                   AND CP.NRLANBOLPAG = BO.NRLANBOLPAG) LOOP
                  V_VRAJUSTE := 0;

                  IF V_VRDISPONIVEL > 0 THEN
                     IF C_CONTAS.VRFINALBPAG > V_VRDISPONIVEL THEN
                        V_VRAJUSTE := ROUND(V_VRDISPONIVEL, 2);
                     ELSE
                        V_VRAJUSTE := ROUND(C_CONTAS.VRFINALBPAG, 2);
                     END IF;

                     V_VRDISPONIVEL := ROUND((V_VRDISPONIVEL - V_VRAJUSTE), 2);
                     /* Verifica se existe ajuste */
                     SELECT COUNT(CDAJUCTPAG)
                       INTO V_EXISTEAJUS
                       FROM AJBOLPAG
                      WHERE CDFILIAL    = C_CONTAS.CDFILIAL
                        AND NRLANBOLPAG = C_CONTAS.NRLANBOLPAG
                        AND CDAJUCTPAG  = V_CDAJUCTPAG;

                     IF V_EXISTEAJUS > 0 THEN
                        UPDATE AJBOLPAG
                           SET VRAJBOLPAG  = ROUND((VRAJBOLPAG + V_VRAJUSTE), 2)
                         WHERE CDFILIAL    = C_CONTAS.CDFILIAL
                           AND NRLANBOLPAG = C_CONTAS.NRLANBOLPAG
                           AND CDAJUCTPAG  = V_CDAJUCTPAG;

                     ELSE
                        INSERT INTO AJBOLPAG
                           (CDAJUCTPAG, VRAJBOLPAG, CDFILIAL, NRLANBOLPAG)
                        VALUES
                           (V_CDAJUCTPAG, V_VRAJUSTE, C_CONTAS.CDFILIAL, C_CONTAS.NRLANBOLPAG);
                     END IF;

                     NOVO_CODIGO_NT('DEVOLUCAOPAG', V_DEVOLUCAOPAG, 1);

                     V_DEVOLUCAOPAG := SUBSTR(V_DEVOLUCAOPAG,11,10);

                     INSERT INTO DEVOLUCAOPAG
                        (NRDEVOLUCAOPAG, CDFILIBOLPAG, NRLANBOLPAG, CDAJUCTPAG, CDFILIAL, NRLANCTONF, VRDEVOLUCAOPAG)
                     VALUES
                        (V_DEVOLUCAOPAG, C_CONTAS.CDFILIAL, C_CONTAS.NRLANBOLPAG, V_CDAJUCTPAG, P_CDFILIAL, P_NRLANCTONF, ROUND(V_VRAJUSTE, 2));

                     UPDATE BOLETPAGAR
                        SET VRFINALBPAG = ROUND((VRFINALBPAG - V_VRAJUSTE), 2)
                      WHERE CDFILIAL    = C_CONTAS.CDFILIAL
                        AND NRLANBOLPAG = C_CONTAS.NRLANBOLPAG;

                     V_VRDEVOLUTI := ROUND((NVL(V_VRDEVOLUTI, 0) + NVL(V_VRAJUSTE, 0)), 2);
                  END IF;
               END LOOP;
            END IF;
         END LOOP;
	  END IF;
  
   ELSIF V_IDENTRSAIDOP = 'E' THEN
      IF NOT V_CDAJUCTREC IS NULL THEN
         FOR C_NOTAS IN (SELECT CDFILIALIT, NRLANCTONFIT
                           FROM ITEMNOTAFISC
                          WHERE CDFILIAL   = P_CDFILIAL
                            AND NRLANCTONF = P_NRLANCTONF) LOOP

            CANCELA_DADOS_FINANCEIRO(C_NOTAS.CDFILIALIT,
                                     C_NOTAS.NRLANCTONFIT,
                                     'N',
                                     V_RETORNO);
            /* RETORNO = 0 - CANCELAMENTO FINANCEIRO CONCLUDO
               RETORNO = 4 - BOLETO COM AJUSTE */
            IF (V_RETORNO = 0 OR V_RETORNO = 4) THEN
               FOR C_CONTAS IN (SELECT BO.CDFILIAL, BO.NRLANBOLREC, ROUND(BO.VRFINALBREC, 2) as VRFINALBREC
                                  FROM CONTARECEBER CR, BOLETRECEBER BO
                                 WHERE CR.CDFILIAL     = C_NOTAS.CDFILIALIT
                                   AND CR.NRLANCTONF   = C_NOTAS.NRLANCTONFIT
                                   AND CR.CDFILIBOLREC = BO.CDFILIAL
                                   AND CR.NRLANBOLREC  = BO.NRLANBOLREC) LOOP
                  V_VRAJUSTE := 0;

                  IF V_VRDISPONIVEL > 0 THEN
                     IF C_CONTAS.VRFINALBREC > V_VRDISPONIVEL THEN
                        V_VRAJUSTE := ROUND(V_VRDISPONIVEL, 2);
                     ELSE
                        V_VRAJUSTE := ROUND(C_CONTAS.VRFINALBREC, 2);
                     END IF;

                     V_VRDISPONIVEL := ROUND((V_VRDISPONIVEL - V_VRAJUSTE), 2);

                     SELECT COUNT(CDAJUCTREC)
                       INTO V_EXISTEAJUS
                       FROM AJBOLREC
                      WHERE CDFILIAL    = C_CONTAS.CDFILIAL
                        AND NRLANBOLREC = C_CONTAS.NRLANBOLREC
                        AND CDAJUCTREC  = V_CDAJUCTREC;

                     IF V_EXISTEAJUS > 0 THEN
                        UPDATE AJBOLREC
                           SET VRAJBOLREC  = ROUND((VRAJBOLREC + V_VRAJUSTE), 2)
                         WHERE CDFILIAL    = C_CONTAS.CDFILIAL
                           AND NRLANBOLREC = C_CONTAS.NRLANBOLREC
                           AND CDAJUCTREC  = V_CDAJUCTREC;

                     ELSE
                        INSERT INTO AJBOLREC
                           (CDAJUCTREC, VRAJBOLREC, CDFILIAL, NRLANBOLREC)
                        VALUES
                           (V_CDAJUCTREC, V_VRAJUSTE, C_CONTAS.CDFILIAL, C_CONTAS.NRLANBOLREC);
                     END IF;

                     NOVO_CODIGO_NT('DEVOLUCAOREC', V_DEVOLUCAOREC, 1);

                     V_DEVOLUCAOREC := SUBSTR(V_DEVOLUCAOREC,11,10);

                     INSERT INTO DEVOLUCAOREC
                        (NRDEVOLUCAOREC, CDFILIBOLREC, NRLANBOLREC, CDAJUCTREC, CDFILIAL, NRLANCTONF, VRDEVOLUCAOREC)
                     VALUES
                        (V_DEVOLUCAOREC, C_CONTAS.CDFILIAL, C_CONTAS.NRLANBOLREC, V_CDAJUCTREC, P_CDFILIAL, P_NRLANCTONF, ROUND(V_VRAJUSTE, 2));

                     UPDATE BOLETRECEBER
                        SET VRFINALBREC = ROUND((VRFINALBREC - V_VRAJUSTE), 2)
                      WHERE CDFILIAL    = C_CONTAS.CDFILIAL
                        AND NRLANBOLREC = C_CONTAS.NRLANBOLREC;

                     V_VRDEVOLUTI := ROUND((NVL(V_VRDEVOLUTI, 0) + NVL(V_VRAJUSTE, 0)), 2);
                  END IF;
               END LOOP;
            END IF;
         END LOOP;
	  END IF;      
   END IF;

   FOR C_NOTAS IN (SELECT DISTINCT CDFILIAL, NRLANCTONF
                     FROM ITEMNOTAFISC
                    WHERE CDFILIAL   = P_CDFILIAL
                      AND NRLANCTONF = P_NRLANCTONF) LOOP

      CANCELA_DADOS_FINANCEIRO(C_NOTAS.CDFILIAL, 
                               C_NOTAS.NRLANCTONF,
                               'S',
                               V_RETORNO);	
									 
      IF V_RETORNO = 0 THEN
         DELETE FROM CONTAPAGAR
          WHERE CDFILIAL   = C_NOTAS.CDFILIAL
            AND NRLANCTONF = C_NOTAS.NRLANCTONF;
      END IF;
   END LOOP;									 									 

   UPDATE NOTAFISCAL
      SET VRDEVOLNFUTL = ROUND((NVL(VRDEVOLNFUTL, 0) + V_VRDEVOLUTI), 2)
    WHERE CDFILIAL     = P_CDFILIAL
      AND NRLANCTONF   = P_NRLANCTONF;
END;
/

-- VERSAO --
BEGIN atuverobj('GERA_AJUSTE_NFDEVOLUCAO','6.49.074'); END;
/

