var AclEngine = function() {
	var READ_ONLY = "R";
	var INVISIBLE = "I";

	var getRuleObj = function(rule){
		rule = rule || INVISIBLE;
		return {
			aclReadOnly: (rule === READ_ONLY),
			aclIsVisible: (rule !== INVISIBLE)
		};
	};

	var extendRules = function (component, parentComponent) {
		component.aclIsVisible = parentComponent.aclIsVisible;
		component.aclReadOnly = parentComponent.aclReadOnly;
	};

	var increaseRuleOfParent = function (component, parentComponent, isZhuAclElementsContainer) {
		if (parentComponent !== null && component.aclIsVisible && !isZhuAclElementsContainer) {
			parentComponent.aclIsVisible = true;
			if(component.aclReadOnly === false) {
				parentComponent.aclReadOnly = false;
			}
		}
	};

	var setAclRules = function (component, parentComponent, aclRules, userGroupId) {
		var aclRule = aclRules[component.id] || aclRules[component.name];
		var rule = (aclRule && aclRule.userGroups[userGroupId]) ?
			aclRule.userGroups[userGroupId].rule : undefined;

		var componentRuleObj;
		if (rule) {
			componentRuleObj = getRuleObj(rule);
		} else {
			componentRuleObj = parentComponent;
		}
		extendRules(component, componentRuleObj);
	};

	var _processMenuAclContainerReadOnly = function (menu, containers) {
		if (menu.aclReadOnly && menu.windowName && containers) {
			if (!containers[menu.windowName]) {
				containers[menu.windowName] = {};
			}
			containers[menu.windowName].isAccessible = false;
		}
	};

	var _processMenuAclMenuChildren = function (menu, aclRules, containers, userGroupId) {
		if (menu.menus && menu.menus.length > 0) {
			var childrenMaxRule = _processMenuAcl(menu.menus, menu, aclRules, containers, userGroupId);
			increaseRuleOfParent(childrenMaxRule, menu);
		}
	};

	var _processMenuAcl = function (menus, parentMenu, aclRules, containers, userGroupId) {
		var maxRules = {};
		menus.forEach(function(menu, index) {
			setAclRules(menu, parentMenu, aclRules, userGroupId);
			_processMenuAclMenuChildren(menu, aclRules, containers, userGroupId);
			increaseRuleOfParent(menu, maxRules);
			_processMenuAclContainerReadOnly(menu, containers);
		});
		return maxRules;
	};

	this.processMenuAcl = function (menus, aclRules, containers, userGroupId, defaultRule) {
		if (aclRules && Array.isArray(menus)) {
			var parentMenu = getRuleObj(defaultRule);
			_processMenuAcl(menus, parentMenu, aclRules, containers, userGroupId);
		}
	};

	var getComponentChildren = function (component) {
		var children = [];
		var childrenTypes = ['widgets', 'fields', 'notifications'];
		if(!(component instanceof Container)){
			childrenTypes.push('actions');
		}
		childrenTypes.forEach(function(type){
			if (component[type]) {
				children = children.concat(component[type]);
			}
		});
		return children;
	};

	var _processAclChildren = function (component, aclRules, userGroupId, isZhuAclElementsContainer) {
		var children = getComponentChildren(component);
		if (children) {
			var maxRules = {};
			for (var i in children) {
				if (children.hasOwnProperty(i)) {
					var childrenRule = _processAcl(children[i], component, aclRules, userGroupId, isZhuAclElementsContainer);
					increaseRuleOfParent(childrenRule, maxRules, isZhuAclElementsContainer);
				}
			}
			increaseRuleOfParent(maxRules, component, isZhuAclElementsContainer);
		}
	};

	var _processAcl = function (component, parentComponent, aclRules, userGroupId, isZhuAclElementsContainer) {
		setAclRules(component, parentComponent, aclRules, userGroupId);
		if (!(component instanceof Field)) _processAclChildren(component, aclRules, userGroupId, isZhuAclElementsContainer);
		return component;
	};

	this.processAcl = function (component, aclRules, userGroupId, defaultRule, isZhuAclElementsContainer) {
		if (aclRules && component) {
			var parentDefault = getRuleObj(defaultRule);
			_processAcl(component, parentDefault, aclRules, userGroupId, isZhuAclElementsContainer);
		}
	};
};